@@ -3,9 +3,11 @@ Array utility functions
 """
 
 from itertools import repeat
+from warnings import warn
+
 import numpy as np
 from numpy.linalg import norm
-from warnings import warn
+
 
 def repeated_array(arr, num, axes = -1):
     """
@@ -28,8 +30,7 @@ def repeated_array(arr, num, axes = -1):
     
     Raises
     ------
-    ValueError
-        If num and axes are tuples of different lengths.
+    ValueError : If num and axes are tuples of different lengths.
     """
     if not num:
         return arr
@@ -48,6 +49,25 @@ def repeated_array(arr, num, axes = -1):
     
     return composite
 
+def complex_array(real, imag):
+    """
+    Combine two real ndarrays into a complex array.
+
+    Parameters
+    ----------
+    real, imag : array_like
+        Real and imaginary parts of a complex array.
+    
+    Returns
+    -------
+    complex : `~numpy.ndarray`
+        Complex array.
+    """
+    real, imag = np.asfarray(real), np.asfarray(imag)
+    comp = real.astype(np.complex)
+    comp += 1j*imag
+    return comp
+
 def mirror(arr, axes = None):
     """ 
     Reverse array over many axes. Generalization of arr[::-1] for many dimensions.
@@ -61,7 +81,8 @@ def mirror(arr, axes = None):
     
     Returns
     -------
-    out : 
+    out : `~numpy.ndarray`
+        Mirrored array.
     """
     if axes is None:
         reverse = [slice(None, None, -1)] * arr.ndim
@@ -156,7 +177,7 @@ def cart2spherical(x, y, z):
     t = np.arccos(z/r)
     return r, p, t
 
-def plane_mesh(v1, v2, x1, x2 = None, origin = [0,0,0]):
+def plane_mesh(v1, v2, x1, x2 = None, origin = (0,0,0)):
     """
     Generate a spatial mesh for a plane defined by two vectors.
 
@@ -189,4 +210,4 @@ def plane_mesh(v1, v2, x1, x2 = None, origin = [0,0,0]):
     xx, yy, zz = tuple(along_v1 * v1[i] + along_v2 * v2[i] for i in range(3))
 
     ox, oy, oz = tuple(origin)
-    return (xx + ox, yy + oy, zz + oz)
\ No newline at end of file
+    return (xx + ox, yy + oy, zz + oz)
