@@ -2,7 +2,9 @@
 """ Electron properties """
 
 import numpy as np
-from scipy.constants import speed_of_light, Planck, elementary_charge, electron_mass
+from scipy.constants import (Planck, electron_mass, elementary_charge,
+                             speed_of_light)
+
 
 def lorentz(keV):
     """
@@ -16,8 +18,12 @@ def lorentz(keV):
     Returns
     -------
     out : array_like or float
+
+    References
+    ----------
+    .. Kirkland 2010 Eq. 2.2
     """
-    return 1/np.sqrt(1 + (elementary_charge*keV*1e3)/(2*electron_mass*speed_of_light**2))
+    return 1 + (elementary_charge*keV*1e3)/(electron_mass*speed_of_light**2)
 
 def electron_wavelength(keV):
     """ 
@@ -36,10 +42,16 @@ def electron_wavelength(keV):
     
     Returns
     -------
-    out : float
-        Electron wavelength [Angs]
+    out : array_like or float
+        Electron wavelength [:math:`Å^{-1}`]
+
+    References
+    ----------
+    .. Kirkland 2010 Eq. 2.5
     """
-    return (Planck/np.sqrt(2*electron_mass*elementary_charge*keV*1e3))*lorentz(keV)*1e10
+    eV = elementary_charge * keV * 1e3
+    wavelength_meters = Planck * speed_of_light / np.sqrt( eV * (2*electron_mass*speed_of_light**2 + eV))
+    return wavelength_meters * 1e10  # wavelength in angstroms
 
 def electron_velocity(keV):
     """
@@ -56,10 +68,17 @@ def electron_velocity(keV):
     
     Returns
     -------
-    out : float
-        Electron velocity [Angs/s]
+    out : array_like or float
+        Electron velocity [:math:`Å / s`]
+
+    References
+    ----------
+    .. Kirkland 2010 Eq. 2.3
     """
-    return Planck/(electron_mass*electron_wavelength(keV))
+    eV = elementary_charge * keV * 1e3
+    m0c2 = electron_mass * speed_of_light**2
+    v_over_c = np.sqrt( eV * (eV + 2 * m0c2)/(m0c2 + eV) )
+    return (c * v_over_c) * 1e10    # speed in Angstroms
 
 def interaction_parameter(keV):
     """
@@ -72,8 +91,8 @@ def interaction_parameter(keV):
     
     Returns
     -------
-    out : float
-        Interaction parameter [rad/(V*Angs)]
+    out : array_like or float
+        Interaction parameter [:math:`rad/(V Å)`]
 
     References
     ----------
