@@ -3,47 +3,44 @@
 Image manipulation of powder diffraction
 ========================================
 """
-from warnings import warn
-
 import numpy as np
+from functools import partial
 
 from .alignment import diff_register
 
+flip = partial(np.rot90, k = 2)
 
 def powder_center(image, mask = None):
-	"""
-	Finds the center of a powder diffraction pattern by comparing the
-	correlation between the input and its image.
+    """
+    Finds the center of a powder diffraction pattern by comparing the
+    correlation between the input and its image.
 
-	Parameters  
-	----------
-	image : `~numpy.ndarray`, ndim 2
-		Image of a powder pattern
-	mask : `~numpy.ndarray` or None, optional
-		Mask of `image`. The mask should evaluate to `True`
-		(or 1) on invalid pixels. If None (default), no mask
-		is used.
+    Parameters  
+    ----------
+    image : `~numpy.ndarray`, ndim 2
+        Image of a powder pattern
+    mask : `~numpy.ndarray` or None, optional
+        Mask of `image`. The mask should evaluate to `True`
+        (or 1) on invalid pixels. If None (default), no mask
+        is used.
 
-	Returns
-	-------
-	center : 2-tuple
-		Center of the powder pattern. The center is returned in the format
-		relevant for array manipulations (center = [row, column] instead of 
-		center = [x,y]).
-	"""
-	if mask is None:
-		mask = np.zeros_like(image, dtype = np.bool)
+    Returns
+    -------
+    center : 2-tuple
+        Center of the powder pattern. The center is returned in the format
+        relevant for array manipulations (center = [row, column] instead of 
+        center = [x,y]).
+    """
+    if mask is None:
+        mask = np.zeros_like(image, dtype = np.bool)
+    mask = np.asarray(mask, dtype = np.bool)
 
-	shift = diff_register(image, np.rot90(image, k = 2), mask = mask * np.rot90(mask, k = 2))
-	midpoints = np.array([int(axis_size / 2) for axis_size in image.shape])
-	
-	# I have found that there is always a residual (0.5, 0.5)
-	center = shift[::-1]/2 + midpoints - np.array([0.5, 0.5])
-	return tuple(center)
+    composite_mask = np.logical_or(mask, flip(mask))
 
-def angular_average(*args, **kwargs):
-    warn('angular_average is deprecated. See azimuthal_average for more features.', DeprecationWarning)
-    return azimuthal_average(*args, **kwargs)
+    shift = diff_register(image, flip(image), mask = composite_mask)
+    midpoints = np.array([int(axis_size / 2) for axis_size in image.shape])
+
+    return tuple(shift[::-1]/2 + midpoints)
 
 def _angle_bounds(bounds):
     b1, b2 = bounds
@@ -57,12 +54,11 @@ def _angle_bounds(bounds):
         b2 -= 360
     return tuple(sorted((b1, b2)))
     
-def azimuthal_average(image, center, mask = None, angular_bounds = None):
+def azimuthal_average(image, center, mask = None, angular_bounds = None, trim = True):
     """
-    This function returns an angularly-averaged pattern computed from a diffraction pattern, 
+    This function returns an azimuthally-averaged pattern computed from an image, 
     e.g. polycrystalline diffraction.
 
-
     Parameters
     ----------
     image : array_like, shape (M, N)
@@ -76,23 +72,24 @@ def azimuthal_average(image, center, mask = None, angular_bounds = None):
         (inclusively) will be used for the average. Angle bounds are specified in degrees.
         0 degrees is defined as the positive x-axis. Angle bounds outside [0, 360) are mapped back
         to [0, 360).
+    trim : bool, optional
+        If True, leading and trailing zeros (possible due to the usage of masks) are trimmed.
 
     Returns
     -------
     radius : `~numpy.ndarray`, ndim 1
-        Radius of the average [px]. If ``mask`` is provided, ``radius`` might not start at one;
-        ``average`` is trimmed of leading invalid pixels.
+        Radius of the average [px]. ``radius`` might not start at zero, depending on the ``trim`` parameter.
     average : `~numpy.ndarray`, ndim 1
         Angular-average of the array.
     """
-    # TODO: interpolation?
-    # TODO: error?
     if mask is None:
         mask = np.zeros_like(image, dtype = np.bool)
 
     xc, yc = center  
 
-    #Create meshgrid and compute radial positions of the data
+    # Create meshgrid and compute radial positions of the data
+    # The radial positions are rounded to the nearest integer
+    # TODO: interpolation? or is that too slow?
     Y, X = np.indices(image.shape)
     R = np.hypot(X - xc, Y - yc)
     Rint = np.rint(R).astype(np.int)
@@ -112,18 +109,15 @@ def azimuthal_average(image, center, mask = None, angular_bounds = None):
     r_bin = np.bincount(Rint, weights = valid)
     radius = np.arange(0, r_bin.size)
 
+    # Make sure r_bin is never 0 since it it used for division anyway
+    np.maximum(r_bin, 1, out = r_bin)
+
     # We ignore the leading and trailing zeroes, which could be due to
-    first, last = _trim_bounds(px_bin)
-    radial_intensity = px_bin[first:last]/r_bin[first:last]
+    first, last = 0, -1
+    if trim:
+        first, last = _trim_bounds(px_bin)
 
-    # Error as the standard error in the mean, at each pixel
-    # Standard error = std / sqrt(N)
-    # std = sqrt(var - mean**2)
-    #if extras is not None:
-    #    var_bin = np.bincount(R, weights = image**2)[first:last]/r_bin[first:last]
-    #    radial_intensity_error = np.sqrt(var_bin - radial_intensity**2)/np.sqrt(r_bin[first:last])
-
-    return radius[first:last], radial_intensity
+    return radius[first:last], px_bin[first:last]/r_bin[first:last]
 
 def _trim_bounds(arr):
     """ Returns the bounds which would be used in numpy.trim_zeros """
@@ -139,4 +133,4 @@ def _trim_bounds(arr):
             break
         else:
             last = last - 1
-    return first, last
+    return first, last
\ No newline at end of file
