@@ -6,34 +6,45 @@ from skimage import data
 from skimage.filters import gaussian
 from skimage.transform import rotate
 
-from .. import shift_image, align, diff_register, ialign
+from .. import shift_image, align, diff_register, ialign, itrack_peak
 from .test_powder import circle_image
 
 np.random.seed(23)
 
 class TestDiffRegister(unittest.TestCase):
 
-	def test_trivial_skimage_data(self):
+	def test_trivial_skimage_data_cropped(self):
 		""" Test that the translation between two identical images is (0,0), even
-		with added random noise and random masks """
+		with added random noise and random masks, with diff_register(..., crop = True) """
 
 		im = np.asfarray(data.camera())
 
 		with self.subTest('No noise'):
-			shift = diff_register(im, im)
+			shift = diff_register(im, im, crop = True)
 			self.assertTrue(np.allclose(shift, (0,0), atol = 1))
 		
 		with self.subTest('With 5% noise'):
 			noise1 = 0.05 * im.max() * np.random.random(size = im.shape)
 			noise2 = 0.05 * im.max() * np.random.random(size = im.shape)
 
-			shift = diff_register(im + noise1, im + noise2)
+			shift = diff_register(im + noise1, im + noise2, crop = True)
+			self.assertTrue(np.allclose(shift, (0,0), atol = 1))
+
+	def test_trivial_skimage_data_no_crop(self):
+		""" Test that the translation between two identical images is (0,0), even
+		with added random noise and random masks, with diff_register(..., crop = False) """
+
+		im = np.asfarray(data.camera())
+
+		with self.subTest('No noise'):
+			shift = diff_register(im, im, crop = False)
 			self.assertTrue(np.allclose(shift, (0,0), atol = 1))
 		
-		with self.subTest('With random masks'):
-			m1 = np.random.choice([True, False], size = im.shape)
+		with self.subTest('With 5% noise'):
+			noise1 = 0.05 * im.max() * np.random.random(size = im.shape)
+			noise2 = 0.05 * im.max() * np.random.random(size = im.shape)
 
-			shift = diff_register(im, im, m1)
+			shift = diff_register(im + noise1, im + noise2, crop = False)
 			self.assertTrue(np.allclose(shift, (0,0), atol = 1))
 	
 	def test_shifted_skimage_data(self):
@@ -60,10 +71,11 @@ class TestDiffRegister(unittest.TestCase):
 			shift = diff_register(im + noise1, im2 + noise2, edge_mask)
 			self.assertTrue(np.allclose(shift, -random_shift, atol = 1))
 		
-		with self.subTest('With random mask'):
-			m1 = np.random.choice([True, False], size = im.shape)
+		with self.subTest('No crop with 10% noise'):
+			noise1 = 0.1 * im.max() * np.random.random(size = im.shape)
+			noise2 = 0.1 * im.max() * np.random.random(size = im.shape)
 
-			shift = diff_register(im, im2, m1)
+			shift = diff_register(im + noise1, im2 + noise2, edge_mask, crop = False)
 			self.assertTrue(np.allclose(shift, -random_shift, atol = 1))
 	
 	def test_side_effects(self):
@@ -76,7 +88,10 @@ class TestDiffRegister(unittest.TestCase):
 		for arr in (im1, im2, mask):
 			arr.setflags(write = False)
 		
-		shift = diff_register(im1, im2, mask)
+		shift = diff_register(im1, im2, mask, crop = True, sigma = 5)
+		shift = diff_register(im1, im2, mask, crop = False, sigma = 5)
+		shift = diff_register(im1, im2, mask, crop = True, sigma = None)
+		shift = diff_register(im1, im2, mask, crop = False, sigma = None)
 
 class TestIAlign(unittest.TestCase):
 
@@ -87,14 +102,32 @@ class TestIAlign(unittest.TestCase):
 		self.assertEqual(len(aligned), 5)
 		self.assertSequenceEqual(data.camera().shape, aligned[0].shape)
 		
-	def test_misaligned_canned_images(self):
+	def test_misaligned_canned_images_fast(self):
 		""" shift images from skimage.data by entire pixels.
 	   We don't expect perfect alignment."""
 		original = data.camera()
 		misaligned = [shift_image(original, (randint(-4, 4), randint(-4, 4))) 
 					  for _ in range(5)]
 
-		aligned = ialign(misaligned, reference = original)
+		aligned = ialign(misaligned, reference = original, fast = True)
+
+		# TODO: find a better figure-of-merit for alignment
+		for im in aligned:
+			# edge will be filled with zeros, we ignore
+			diff = np.abs(original[5:-5, 5:-5] - im[5:-5, 5:-5])
+
+			# Want less than 1% difference
+			percent_diff = np.sum(diff) / (diff.size * (original.max() - original.min()))
+			self.assertLess(percent_diff, 1)
+
+	def test_misaligned_canned_images_notfast(self):
+		""" shift images from skimage.data by entire pixels.
+	   We don't expect perfect alignment."""
+		original = data.camera()
+		misaligned = [shift_image(original, (randint(-4, 4), randint(-4, 4))) 
+					  for _ in range(5)]
+
+		aligned = ialign(misaligned, reference = original, fast = False)
 
 		# TODO: find a better figure-of-merit for alignment
 		for im in aligned:
@@ -148,13 +181,13 @@ class TestAlign(unittest.TestCase):
 		aligned = align(im, reference = im, fill_value = np.nan)
 		self.assertEqual(im.dtype, data.camera().dtype)
 
-	def test_misaligned_canned_images(self):
+	def test_misaligned_canned_images_fast(self):
 		""" shift images from skimage.data by entire pixels.
 	   	We don't expect perfect alignment."""
 		original = data.camera()
 		misaligned = shift_image(original, (randint(-4, 4), randint(-4, 4))) 
 
-		aligned = align(misaligned, reference = original)
+		aligned = align(misaligned, reference = original, fast = True)
 
 		# edge will be filled with zeros, we ignore
 		diff = np.abs(original[5:-5, 5:-5] - aligned[5:-5, 5:-5])
@@ -163,5 +196,41 @@ class TestAlign(unittest.TestCase):
 		percent_diff = np.sum(diff) / (diff.size * (original.max() - original.min()))
 		self.assertLess(percent_diff, 1)
 
+	def test_misaligned_canned_images_notfast(self):
+		""" shift images from skimage.data by entire pixels.
+	   	We don't expect perfect alignment."""
+		original = data.camera()
+		misaligned = shift_image(original, (randint(-4, 4), randint(-4, 4))) 
+
+		aligned = align(misaligned, reference = original, fast = False)
+
+		# edge will be filled with zeros, we ignore
+		diff = np.abs(original[5:-5, 5:-5] - aligned[5:-5, 5:-5])
+
+		# Want less than 1% difference
+		percent_diff = np.sum(diff) / (diff.size * (original.max() - original.min()))
+		self.assertLess(percent_diff, 1)
+
+class TestItrackPeak(unittest.TestCase):
+
+    def test_trivial(self):
+        """ Test that shift is identically zero for images that are identical """
+        # Array prototype is just zeros
+        # with a 'peak' in the center
+        prototype = np.zeros(shape = (17, 17))
+        prototype[9,9] = 10
+        images = [np.array(prototype) for _ in range(20)]
+        shifts = itrack_peak(images, row_slice = np.s_[:], col_slice = np.s_[:])
+
+        for shift in shifts:
+            self.assertTrue(np.allclose(shift, (0.0, 0.0)))
+
+    def test_length(self):
+        """ Test that shifts yielded by itrack_peak are as numerous as the number of input pictures """
+        images = [np.random.random(size = (4,4)) for _ in range(20)]
+        shifts = list(itrack_peak(images, row_slice = np.s_[:], col_slice = np.s_[:]))
+
+        self.assertEqual(len(shifts), len(images))
+
 if __name__ == '__main__':
 	unittest.main()
\ No newline at end of file
