@@ -8,13 +8,13 @@ import unittest
 class TestPowdersim(unittest.TestCase):
 
 	def setUp(self):
-		self.scattering_length = np.linspace(0.11, 0.8, 200)
+		self.q = np.linspace(2, 10, 200)
 		self.crystal = Crystal.from_database('C')
 
 	def test_return_shape(self):
 		""" Test that the return shape of powdersim() is as expected """
-		pattern = powdersim(self.crystal, self.scattering_length)
-		self.assertSequenceEqual(pattern.shape, self.scattering_length.shape)
+		pattern = powdersim(self.crystal, self.q)
+		self.assertSequenceEqual(pattern.shape, self.q.shape)
 
 if __name__ == '__main__':
 	unittest.main()
\ No newline at end of file
