@@ -2,17 +2,20 @@
 from copy import deepcopy as copy
 from functools import lru_cache
 from glob import glob
-from os.path import basename, dirname, isdir, isfile, join
+from itertools import islice
+from os import mkdir
+from pathlib import Path
 from urllib.request import urlretrieve
 
 import numpy as np
 from spglib import (find_primitive, get_error_message, get_spacegroup_type,
                     get_symmetry_dataset)
 
-from . import Atom, CIFParser, Lattice, PDBParser
+from . import Atom, AtomicStructure, CIFParser, Lattice, PDBParser
 from .. import affine_map
+from .parsers import CIFParser, CODParser, PDBParser
 
-CIF_ENTRIES = glob(join(dirname(__file__), 'cifs', '*.cif'))
+CIF_ENTRIES = frozenset( (Path(__file__).parent / 'cifs').glob('*.cif') )
 
 def symmetry_expansion(atoms, symmetry_operators):
     """
@@ -43,10 +46,9 @@ def symmetry_expansion(atoms, symmetry_operators):
             uniques.add(new)
     yield from uniques
 
-class Crystal(Lattice):
+class Crystal(AtomicStructure, Lattice):
     """
-    This object is the basis for inorganic crystals such as VO2, 
-    and protein crystals such as bR. 
+    The :class:`Crystal` class is a set-like container that represent crystalline structures. 
 
     In addition to constructing the ``Crystal`` object yourself, other constructors
     are also available (and preferred):
@@ -66,40 +68,18 @@ class Crystal(Lattice):
     unitcell : iterable of ``Atom``
         Unit cell atoms. It is assumed that the atoms are in fractional coordinates.
     lattice_vectors : iterable of array_like
-        Lattice vectors.
+        Lattice vectors. If ``lattice_vectors`` is provided as a 3x3 array, it 
+        is assumed that each lattice vector is a row.
     source : str or None, optional
-        Provenance, e.g. filename.
+        Provenance, e.g. filename. Only used for bookkeeping.
     """
 
-    builtins = frozenset(map(lambda fn: basename(fn).split('.')[0], CIF_ENTRIES))
+    builtins = frozenset(map(lambda fn: fn.stem, CIF_ENTRIES))
 
     def __init__(self, unitcell, lattice_vectors, source = None, **kwargs):
-        self.unitcell = frozenset(unitcell)
+        super().__init__(atoms = unitcell, lattice_vectors = lattice_vectors, **kwargs)
         self.source = source
-        super().__init__(lattice_vectors, **kwargs)
     
-    def __iter__(self):
-        yield from iter(self.unitcell)
-    
-    def __len__(self):
-        return len(self.unitcell)
-    
-    def __repr__(self):
-        return '< Crystal object with unit cell of {} atoms >'.format(len(self))
-    
-    def __eq__(self, other):
-        return (isinstance(other, self.__class__) 
-                and (set(self) == set(other))
-                and super().__eq__(other))
-    
-    def __array__(self):
-        """ Returns an array in which each row represents a unit cell atom """
-        arr = np.empty(shape = (len(self), 4), dtype = np.float)
-        for row, atm in enumerate(self):
-            arr[row, 0] = atm.atomic_number
-            arr[row, 1:] = atm.coords
-        return arr
-
     @classmethod
     @lru_cache(maxsize = len(builtins), typed = True) # saves a lot of time in tests
     def from_cif(cls, path):
@@ -108,7 +88,7 @@ class Crystal(Lattice):
 
         Parameters
         ----------
-        path : str
+        path : path-like
             File path
         
         References
@@ -128,14 +108,14 @@ class Crystal(Lattice):
 
         Parameters
         ----------
-        name : str
+        name : path-like
             Name of tne databse entry. Available items can be retrieved from `Crystal.builtins`
         """
         if name not in cls.builtins:
             raise ValueError('Entry {} is not available in the database. See \
                               Crystal.builtins for valid entries.'.format(name))
         
-        path = join(dirname(__file__), 'cifs', name + '.cif')
+        path = Path(__file__).parent / 'cifs' / (name + '.cif')
         return cls.from_cif(path)
     
     @classmethod
@@ -155,25 +135,10 @@ class Crystal(Lattice):
             Whether or not to overwrite files in cache if they exist. If no revision 
             number is provided, files will always be overwritten. 
         """
-        if revision is None:
-            overwrite = True
-        
-        if not isdir(download_dir):
-            os.mkdir(download_dir)
-        
-        url = 'http://www.crystallography.net/cod/{}.cif'.format(num)
-
-        if revision is not None:
-            url = url + '@' + str(revision)
-            base = '{iden}-{rev}.cif'.format(iden = num, rev = revision)
-        else:
-            base = '{}.cif'.format(num)
-        path = join(download_dir, base)
-
-        if (not isfile(path)) or overwrite:
-            urlretrieve(url, path)
-        
-        return cls.from_cif(path)
+        with CODParser(num, revision, download_dir, overwrite) as parser:
+            return cls(unitcell = symmetry_expansion(parser.atoms(), parser.symmetry_operators()),
+                       lattice_vectors = parser.lattice_vectors(),
+                       source = 'COD num:{n} rev:{r}'.format(n = num, r = revision))
 
     @classmethod
     def from_pdb(cls, ID, download_dir = 'pdb_cache', overwrite = False):
@@ -191,10 +156,10 @@ class Crystal(Lattice):
             Whether or not to overwrite files in cache if they exist. If no revision 
             number is provided, files will always be overwritten. 
         """
-        parser = PDBParser(ID = ID, download_dir = download_dir)
-        return cls(unitcell = symmetry_expansion(parser.atoms(), parser.symmetry_operators()),
-                   lattice_vectors = parser.lattice_vectors(),
-                   source = str(parser.file))
+        with PDBParser(ID = ID, download_dir = download_dir) as parser:
+            return cls(unitcell = symmetry_expansion(parser.atoms(), parser.symmetry_operators()),
+                       lattice_vectors = parser.lattice_vectors(),
+                       source = parser.filename)
     
     @classmethod
     def from_ase(cls, atoms):
@@ -210,6 +175,38 @@ class Crystal(Lattice):
         
         return cls(unitcell = [Atom.from_ase(atm) for atm in atoms], 
                    lattice_vectors = lattice_vectors)
+    
+    # TODO: test against known XYZ file
+    def write_xyz(self, fname, comment = None):
+        """
+        Generate an atomic coordinates .xyz file from a crystal structure.
+
+        Parameters
+        ----------
+        fname : path-like
+            The XYZ file will be written to this file. If the file already exists,
+            it will be overwritten.
+        comment : str or None, optional
+            Comment to include at the second line of ``fname``.
+        """
+        # Format is specified here:
+        #   http://openbabel.org/wiki/XYZ_%28format%29
+        comment = comment or ''
+        atom_format_str = '  {:<2}       {:10.5f}       {:10.5f}       {:10.5f}'
+
+        with open(fname, 'wt', encoding = 'ascii') as file:
+            # First two lines are:
+            #   1. Number of atoms described in the file
+            #   2. Optional comment
+            file.write(str(len(self)) + '\n')
+            file.write(comment + '\n')
+
+            # Write atomic data row-by-row
+            # For easier human readability, atoms are sorted
+            # by element
+            for atom in self.itersorted():
+                row = atom_format_str.format(atom.element, *atom.xyz(self))
+                file.write(row + '\n')
 
     def _spglib_cell(self):
         """ Returns an array in spglib's cell format. """
@@ -233,8 +230,7 @@ class Crystal(Lattice):
 
         Raises
         ------
-        RuntimeError
-            If primitive cell could not be found.
+        RuntimeError : If primitive cell could not be found.
         
         Notes
         -----
@@ -271,8 +267,7 @@ class Crystal(Lattice):
         
         Raises
         ------
-        ImportError
-            If ASE is not installed
+        ImportError : If ASE is not installed
         """
         from ase import Atoms
         
@@ -313,8 +308,7 @@ class Crystal(Lattice):
         
         Raises
         ------
-        RuntimeError
-            If symmetry-determination has yielded an error.
+        RuntimeError : If symmetry-determination has yielded an error.
         
         Notes
         -----
@@ -326,18 +320,55 @@ class Crystal(Lattice):
                                        angle_tolerance = angle_tolerance)
 
         if dataset: 
-            info = dict()
-            info.update( {'international_symbol': dataset['international'],
-                          'hall_symbol': dataset['hall'],
-                          'international_number': dataset['number'],
-                          'hall_number': dataset['hall_number']} )
+            spg_type = get_spacegroup_type(dataset['hall_number'])
+
+            info = {'international_symbol': dataset['international'],
+                    'hall_symbol'         : dataset['hall'],
+                    'international_number': dataset['number'],
+                    'hall_number'         : dataset['hall_number'],
+                    'international_full'  : spg_type['international_full'],
+                    'pointgroup'          : spg_type['pointgroup_international']} 
+
+            err_msg = get_error_message()
+            if (err_msg != 'no error'):
+                raise RuntimeError('Symmetry-determination has returned the following error: {}'.format(err_msg))
             
-            spg_type = get_spacegroup_type(info['hall_number'])
-            info.update( {'international_full': spg_type['international_full'],
-                          'pointgroup': spg_type['pointgroup_international']} )
-
             return info
+        
+        return None
 
-        err_msg = get_error_message()
-        if err_msg:
-            raise RuntimeError('Symmetry-determination has returned the following error: {}'.format(err_msg))
+    def __str__(self):
+        """ String representation of this instance. Atoms may be omitted. """
+        return self._to_string(natoms = 10)
+    
+    def __repr__(self):
+        """ Verbose string representation of this instance. """
+        return self._to_string(natoms = None)
+    
+    def _to_string(self, natoms = None):
+        """ Generate a string representation of this Crystal. Only include a maximum of `natoms` if provided. """
+        if natoms is not None:
+            if natoms >= len(self):
+                return self._to_string(natoms = None)
+
+        # Note : Crystal subclasses need not override this method
+        # since the class name is dynamically determined
+        rep = '< {clsname} object with following unit cell:'.format(clsname = self.__class__.__name__)
+
+        # For very large structures, listing all atoms is prohibitive
+        if natoms is None:
+            atoms = self.itersorted()
+        else:
+            atoms = islice(self.itersorted(), natoms)
+
+        # Note that repr(Atom(...)) includes these '< ... >'
+        # We remove those for cleaner string representation
+        for atm in atoms:
+            rep += '\n    ' + repr(atm).replace('<', '').replace('>', '').strip()
+        
+        if natoms is not None:
+            rep += '\n      ... omitting {:d} atoms ...'.format(len(self) - natoms) 
+
+        rep += '\nLattice parameters: \n    {:.3f}Å, {:.3f}Å, {:.3f}Å, {:.2f}°, {:.2f}°, {:.2f}°'.format(*self.lattice_parameters)
+        rep += '\nSource: \n    {} >'.format(self.source or 'N/A')
+        return rep
\ No newline at end of file
