@@ -1,20 +1,26 @@
 # -*- coding: utf-8 -*-
-from math import radians
-from copy import deepcopy, copy
-from random import choice, seed
-from itertools import permutations
-import numpy as np
-from .. import Crystal, Atom, Lattice
-from ... import rotation_matrix, transform
+import pickle
+import tempfile
 import unittest
+from copy import copy, deepcopy
+from itertools import permutations
+from math import radians
+from pathlib import Path
+from random import choice, seed
+
+import numpy as np
+
+from .. import Atom, Crystal, Lattice
+from ... import rotation_matrix, transform
 
 #seed(23)
 
 try:
     import ase
-    ASE = True
 except ImportError:
     ASE = False
+else:
+    ASE = True
 
 @unittest.skipIf(not ASE, 'ASE not importable')
 class TestAseAtoms(unittest.TestCase):
@@ -38,17 +44,6 @@ class TestAseAtoms(unittest.TestCase):
         # self.assertSetEqual(set(self.crystal), set(crystal2))
         self.assertEqual(len(self.crystal), len(crystal2))
 
-class TestCrystalMethods(unittest.TestCase):
-
-    def setUp(self):
-        name = choice(list(Crystal.builtins))
-        self.crystal = Crystal.from_database(name)
-    
-    def test_array(self):
-        """ Test Crystal.__array__ """
-        arr = np.array(self.crystal)
-        self.assertSequenceEqual(arr.shape, (len(self.crystal), 4))
-
 class TestSpglibMethods(unittest.TestCase):
     
     def test_spacegroup_info_graphite(self):
@@ -82,6 +77,20 @@ class TestSpglibMethods(unittest.TestCase):
                 prim2 = prim.primitive(symprec = 0.1)
                 self.assertIs(prim, prim2)
 
+class TestCrystalSpecialMethods(unittest.TestCase):
+
+    def test_str_vs_repr(self):
+        """ Test that str and repr are workign as expected """
+        for name in Crystal.builtins:
+            with self.subTest(name):
+                c = Crystal.from_database(name)
+
+                # If small crystal, repr and str should be the same
+                if len(c) <= 10:
+                    self.assertEqual(repr(c), str(c))
+                else:
+                    self.assertNotEqual(repr(c), str(c))
+
 class TestCrystalRotations(unittest.TestCase):
 
     def setUp(self):
@@ -147,17 +156,27 @@ class TestCrystalConstructors(unittest.TestCase):
     
     def test_from_pdb(self):
         """ Test Crystal.from_pdb constructor """
-        # the tests on PDBParser are also using the test_cache folder
-        c = Crystal.from_pdb('1fbb', download_dir = 'test_cache')
-        self.assertIn('1fbb', c.source)
+        with tempfile.TemporaryDirectory() as temp_dir:
+            # the tests on PDBParser are also using the test_cache folder
+            c = Crystal.from_pdb('1fbb', download_dir = temp_dir)
+            self.assertIn('1fbb', c.source)
     
     def test_from_cod(self):
         """ Test building a Crystal object from the COD """
-        # revision = None and latest revision should give the same Crystal
-        c = Crystal.from_cod(1521124, download_dir = 'test_cache')
-        c2 = Crystal.from_cod(1521124, revision = 176429, download_dir = 'test_cache')
+        with tempfile.TemporaryDirectory() as temp_dir:
+            # revision = None and latest revision should give the same Crystal
+            c = Crystal.from_cod(1521124, download_dir = temp_dir)
+            c2 = Crystal.from_cod(1521124, revision = 176429, download_dir = temp_dir)
 
-        self.assertEqual(c, c2)
+        self.assertEqual(c, c2)     
+
+    def test_from_cod_new_dir(self):     
+        """ Test that a cache dir is created by Crystal.from_cod """
+        with tempfile.TemporaryDirectory() as temp_dir:
+            download_dir = Path(temp_dir) / 'test_cod'
+            self.assertFalse(download_dir.exists())
+            c = Crystal.from_cod(1521124, download_dir = download_dir)
+            self.assertTrue(download_dir.exists())
 
 if __name__ == '__main__':
-    unittest.main()
\ No newline at end of file
+    unittest.main()
