@@ -1,36 +1,37 @@
 # -*- coding: utf-8 -*-
 import os
+import tempfile
 import unittest
-from spglib import get_symmetry_dataset
 from warnings import filterwarnings
 
 import numpy as np
+from spglib import get_symmetry_dataset
 
 from .. import Crystal
 from ... import transform
-from ..cif_parser import CIFParser
-from ..pdb_parser import PDBParser
+from ..parsers import CIFParser, PDBParser
 from ..spg_data import Hall2Number
 
 filterwarnings('ignore', category = UserWarning)
 
 class TestPDBParser(unittest.TestCase):
 
-    def setUp(self):
-        self.parser = PDBParser('1fbb', download_dir = 'test_cache')
-    
     def test_fractional_atoms(self):
-        """ Test the CIFParser returns fractional atomic coordinates. """
-        for atm in self.parser.atoms():
-            self.assertLessEqual(atm.coords.max(), 1)
-            self.assertGreaterEqual(atm.coords.min(), 0)
+        """ Test the PDBParser returns fractional atomic coordinates. """
+        with tempfile.TemporaryDirectory() as temp_dir:
+            with PDBParser('1fbb', download_dir = temp_dir) as parser:
+                for atm in parser.atoms():
+                    self.assertLessEqual(atm.coords.max(), 1)
+                    self.assertGreaterEqual(atm.coords.min(), 0)
         
     def test_symmetry_operators(self):
         """ Test that the non-translation part of the symmetry_operators is an invertible
         matrix of determinant 1 | -1 """
-        for sym_op in self.parser.symmetry_operators():
-            t = sym_op[:3,:3]
-            self.assertAlmostEqual(abs(np.linalg.det(t)), 1, places = 5)
+        with tempfile.TemporaryDirectory() as temp_dir:
+            with PDBParser('1fbb', download_dir = temp_dir) as parser:
+                for sym_op in parser.symmetry_operators():
+                    t = sym_op[:3,:3]
+                    self.assertAlmostEqual(abs(np.linalg.det(t)), 1, places = 5)
 
 class TestCIFParser(unittest.TestCase):
     """ Test the CIFParser on all CIF files stored herein """
@@ -93,7 +94,7 @@ class TestCIFParser(unittest.TestCase):
         self.assertEqual(len(vo2), 12)
         self.assertSequenceEqual(vo2.lattice_parameters, 
                                  (5.7430000000000003, 4.5170000000000003, 5.375, 90.0, 122.60000000000001, 90.0))
-        self.assertAlmostEqual(vo2.volume, 117.4661530) # from cif2cell
+        self.assertAlmostEqual(vo2.volume, 117.4661530) # from cif2cell   
 
 if __name__ == '__main__':
     unittest.main()
